const express = require('express');
const multer = require('multer');
const coupon_master = express();
// Middleware for parsing JSON
coupon_master.use(express.json());
const connection = require('../config');
// Configure multer for file uploads
//const upload = multer();
const upload = multer({ storage:multer.memoryStorage() }); //Define a directory to store uploaded files

//Get method
coupon_master.get('/coupon_master', (req, res) => {
  const query = 'SELECT * FROM coupon_master';

  connection.query(query, (err, results) => {
    if (err) {
      console.error('Error executing MySQL query: ' + err.stack);
      res.status(500).json({ error: 'Failed to retrieve coupon' });
      return;
    }

    if (results.length === 0) {
      res.status(404).json({ error: 'No coupon found' });
      return;
    }

    // Function to get total_available_qty and consumed_qty for a coupon
    const getQtyInfo = (coupon, callback) => {
      const id = coupon.id;

      // Query coupon_details to count rows with matching master_id and is_assigned = 1
      const consumedQtyQuery = 'SELECT COUNT(*) AS consumed_qty FROM coupon_details WHERE master_id = ? AND is_assigned = 1';
      
      // Query coupon_details to count rows with matching master_id (total available quantity)
      const totalQtyQuery = 'SELECT COUNT(*) AS total_available_qty FROM coupon_details WHERE master_id = ?';

      connection.query(consumedQtyQuery, [id], (err, consumedQtyResults) => {
        if (err) {
          console.error('Error executing MySQL query: ' + err.stack);
          callback(err, null);
        } else {
          const consumedQuantity = consumedQtyResults[0].consumed_qty;

          connection.query(totalQtyQuery, [id], (err, totalQtyResults) => {
            if (err) {
              console.error('Error executing MySQL query: ' + err.stack);
              callback(err, null);
            } else {
              const totalAvailableQty = totalQtyResults[0].total_available_qty;

              // Add the total_available_qty and consumed_qty to the coupon object
              coupon.total_available_qty = totalAvailableQty;
              coupon.consumed_qty = consumedQuantity;

              callback(null, coupon);
            }
          });
        }
      });
    };

    // Process each coupon individually and get the total_available_qty and consumed_qty
    const processCoupons = (index) => {
      if (index < results.length) {
        getQtyInfo(results[index], (err, couponWithQtyInfo) => {
          if (!err) {
            // Replace the original coupon with the one containing the quantity information
            results[index] = couponWithQtyInfo;
          }

          // Continue processing the next coupon
          processCoupons(index + 1);
        });
      } else {
        // All coupons have been processed, send the response
        res.json(results);
      }
    };

    // Start processing coupons from the first one
    processCoupons(0);
  });
});





//get by id
coupon_master.get('/coupon_master/:id', (req, res) => {   //retreiving all the shop names where created_by
  const id = req.params.id;

  // Execute the SQL query to retrieve shop names for the provided created_by user ID
  const query = 'SELECT * FROM coupon_master WHERE id = ?';
  connection.query(query, [id], (err, result) => {
    if (err) {
      console.error('Error retrieving Mysql query:', err);
      return res.status(500).json({ error: 'Internal server error' });
    }
    return res.json(result);
  });
});

// POST endpoint for uploading a coupon with an image
coupon_master.post('/upload', upload.single('coupon_image'), (req, res) => {
  // Check if req.file exists before accessing its properties
  if (!req.file) {
    return res.status(400).json({ error: 'No file uploaded' });
  }

  const { valid_start_date, valid_end_date } = req.body;
  const imageFilePath = req.file.buffer.toString('base64');

  // Insert file information into the MySQL database
  const query = 'INSERT INTO coupon_master (coupon_image, valid_start_date, valid_end_date, created_at, updated_at) VALUES ( ?, ?, ?,  NOW(), NOW())';
  
  connection.query(query, [imageFilePath, valid_start_date, valid_end_date], (err, result) => {
    if (err) {
      console.error('Error inserting data into the database: ', err);
      return res.status(500).json({ error: 'Internal Server Error' });
    }

    // Get the ID of the newly inserted record
    const insertedId = result.insertId;

    // Create a response object with all the data, including the ID
    const responseData = {
      id: insertedId,
      coupon_image: imageFilePath,
      valid_start_date,
      valid_end_date,
      created_at: new Date().toISOString(), // Proper date format
      updated_at: new Date().toISOString(), // Proper date format
    };

    res.status(200).json(responseData); // 200 status code indicates resource creation
  });
});

  
//update coupon_master
coupon_master.put('/update/:id', upload.single('coupon_image'), (req, res) => {
    const couponId = req.params.id; // Get the ID from the URL parameter
  
    const { valid_start_date, valid_end_date, total_available_qty } = req.body;
    const imageFilePath = req.file.buffer.toString('base64'); // This is the path to the uploaded image file
  
    // Update file information in the MySQL database
    const query = 'UPDATE coupon_master SET coupon_image=?, valid_start_date=?, valid_end_date=?, total_available_qty=?,  updated_at=NOW() WHERE id=?';
    connection.query(query, [imageFilePath, valid_start_date, valid_end_date, total_available_qty,  couponId], (err, result) => {
      if (err) {
        console.error('Error updating data in the database: ', err);
        return res.status(500).json({ error: 'Internal Server Error' });
      }
  
      if (result.affectedRows === 0) {
        // No rows were updated, which means the record with the given ID was not found
        return res.status(404).json({ error: 'Coupon not found' });
      }
  
      // Create a response object with the updated data
      const responseData = {
        id: couponId,
        coupon_image: imageFilePath,
        valid_start_date,
        valid_end_date,
        total_available_qty,
        updated_at: new Date(), // You can use a proper date format here
      };
  
      res.status(200).json(responseData); // 200 status code indicates success
    });
  });
  
// DELETE method for deleting a coupon by ID
coupon_master.delete('/delete/:id', (req, res) => {
    const couponId = req.params.id; // Get the ID from the URL parameter

    // Delete the coupon from the MySQL database
    const deleteQuery = 'DELETE FROM coupon_master WHERE id = ?';

    connection.query(deleteQuery, [couponId], (err, result) => {
        if (err) {
            console.error('Error deleting data from the database: ', err);
            return res.status(500).json({ error: 'Internal Server Error' });
        }

        if (result.affectedRows === 0) {
            // No rows were deleted (coupon with given ID not found)
            return res.status(404).json({ error: 'Coupon not found' });
        }

        res.status(200).json({ message: 'Coupon successfully deleted', deletedId: couponId });
    });
});


module.exports = coupon_master;